"""
Authors: Peter Mawhorter
Consulted:
Date: 2023-10-22
Purpose: Testing code for weddingSeating function.
"""

import optimism

# Imports your module
import weddingSeating

# Testing variables
julia = {'name': 'Julia', 'hates': [], 'dislikes': []}
yasmin = {'name': 'Yasmin', 'hates': [], 'dislikes': [julia]}
aaron = {'name': 'Aaron', 'hates': [], 'dislikes': []}
konrad = {'name': 'Konrad', 'hates': [aaron]}
sigmund = {'name': 'Sigmund'}
bellas = {'name': 'Bellas'}
rory = {'name': 'Rory', 'dislikes': [julia]}
naia = {'name': 'Naia'}
thomas = {'name': 'Thomas', 'hates': [konrad]}
ceres = {'name': 'Ceres', 'dislikes': [naia]}
zeke = {'name': 'Zeke'}
lulu = {'name': 'Lulu', 'hates': [thomas]}
evan = {'name': 'Evan', 'hates': [julia, yasmin], 'dislikes': [rory]}
zara = {'name': 'Zara', 'dislikes': [julia]}
amara = {'name': 'Amara', 'hates': [julia]}
evan['dislikes'].append(zara)
evan['dislikes'].append(amara)
zara['hates'] = [amara]
xiaohua = {'name': 'Xiaohua', 'hates': [julia]}
yoth = {'name': 'Yoth'}
vanessa = {'name': 'Vanessa', 'dislikes': [julia, yasmin]}
hammond = {'name': 'Hammond', 'dislikes': [ceres, rory, yoth, naia]}


tester = optimism.testFunctionMaybe(weddingSeating, "weddingSeating")

# This is a very simple test case showing the basic hatred mechanism. It
# also does not test missing dictionary keys
case1 = tester.case([julia, yasmin, aaron], 3)
case1.checkReturnedStructure([[aaron, julia], [yasmin]])

# This is a moderately complex test case showing off a mix of different
# scenarios
case2 = tester.case(
    [
        julia,
        aaron,
        yasmin,
        konrad,
        sigmund,
        bellas,
        rory,
        naia,
        thomas,
        ceres,
        zeke,
        lulu
    ],
    5
)
case2.checkReturnedStructure(
    [
        [bellas, rory, sigmund, aaron, julia],
        [zeke, naia, konrad, ceres, yasmin],
        [thomas],
        [lulu]
    ]
)

# This test case checks for some multiple-skip scenarios due to hatred,
# dislike, and full tables
case3 = tester.case(
    [
        julia,
        yasmin,
        rory,
        evan,
        zara,
        amara,
        xiaohua
    ],
    2
)
case3.checkReturnedStructure(
    [
        [julia],
        [rory, yasmin],
        [xiaohua, evan],
        [zara],
        [amara]
    ]
)

# This case checks that tables fill up properly when everyone can sit
# almost anywhere
case4 = tester.case(
    [
        julia,
        aaron,
        sigmund,
        bellas,
        naia,
        zeke,
        yoth
    ],
    3
)
case4.checkReturnedStructure(
    [
        [sigmund, aaron, julia],
        [zeke, naia, bellas],
        [yoth]
    ]
)

# This test case checks that you've implemented dislikes properly, since
# almost everyone can sit at the same table but in many cases they cannot
# sit in the first available slot.
case5 = tester.case(
    [
        julia,
        naia,
        yoth,
        ceres,
        yasmin,
        vanessa,
        rory,
        zara,
        hammond
    ],
    10
)
case5.checkReturnedStructure(
    [
        [ceres, zara, rory, yasmin, yoth, vanessa, naia, julia],
        [hammond]
    ]
)
